# Copyright (C) 2016 Barry Warsaw
#
# This project is licensed under the terms of the Apache 2.0 License.  See
# LICENSE.txt for details.

"""@public -- populate __all__"""

from public.public import public as py_public
try:
    from _public import public as c_public
except ImportError:                                 # pragma: nocover
    # This library was built without the extension module.
    c_public = None


__version__ = '0.5'


if c_public is None:                                # pragma: nocover
    py_public(public=py_public)
    py_public(py_public=py_public)
else:                                               # pragma: nocover
    c_public(public=c_public)
    c_public(py_public=py_public)


def install():
    """Install @public into builtins."""
    import builtins
    builtins.public = c_public or py_public



# Copyright (C) 2016 Barry Warsaw
#
# This project is licensed under the terms of the Apache 2.0 License.  See
# LICENSE.txt for details.

"""Pure-Python implementation."""

import sys


# http://bugs.python.org/issue26632
def public(thing=None, **kws):
    mdict = (sys._getframe(1).f_globals
             if thing is None
             else sys.modules[thing.__module__].__dict__)
    dunder_all = mdict.setdefault('__all__', [])
    if not isinstance(dunder_all, list):
        raise ValueError(
            '__all__ must be a list not: {}'.format(type(dunder_all)))
    if thing is not None:
        dunder_all.append(thing.__name__)
    for key, value in kws.items():
        dunder_all.append(key)
        mdict[key] = value
    return thing



# Copyright (C) 2016 Barry Warsaw
#
# This project is licensed under the terms of the Apache 2.0 License.  See
# LICENSE.txt for details.

import os
import sys
import builtins
import unittest

try:
    from _public import public as c_public
except ImportError:
    # This library was built without the extension module.
    c_public = None
from contextlib import ExitStack, contextmanager
from importlib import import_module
from public import install
from public.public import public as py_public
from tempfile import TemporaryDirectory


@contextmanager
def syspath(directory):
    try:
        sys.path.insert(0, directory)
        yield
    finally:
        assert sys.path[0] == directory
        del sys.path[0]


@contextmanager
def sysmodules():
    modules = sys.modules.copy()
    try:
        yield
    finally:
        sys.modules = modules


class TestPublic(unittest.TestCase):
    import_line = 'from public import public'

    def setUp(self):
        self.resources = ExitStack()
        self.addCleanup(self.resources.close)
        self.tmpdir = self.resources.enter_context(TemporaryDirectory())
        self.resources.enter_context(syspath(self.tmpdir))
        self.resources.enter_context(sysmodules())
        self.modpath = os.path.join(self.tmpdir, 'example.py')

    def test_atpublic_function(self):
        with open(self.modpath, 'w', encoding='utf-8') as fp:
            print("""\
{}

@public
def a_function():
    pass
""".format(self.import_line), file=fp)
        module = import_module('example')
        self.assertEqual(module.__all__, ['a_function'])

    def test_atpublic_function_runnable(self):
        with open(self.modpath, 'w', encoding='utf-8') as fp:
            print("""\
{}

@public
def a_function():
    return 1
""".format(self.import_line), file=fp)
        module = import_module('example')
        self.assertEqual(module.a_function(), 1)

    def test_atpublic_class(self):
        with open(self.modpath, 'w', encoding='utf-8') as fp:
            print("""\
{}

@public
class AClass:
    pass
""".format(self.import_line), file=fp)
        module = import_module('example')
        self.assertEqual(module.__all__, ['AClass'])

    def test_atpublic_class_runnable(self):
        with open(self.modpath, 'w', encoding='utf-8') as fp:
            print("""\
{}

@public
class AClass:
    pass
""".format(self.import_line), file=fp)
        module = import_module('example')
        self.assertIsInstance(module.AClass(), module.AClass)

    def test_atpublic_two_things(self):
        with open(self.modpath, 'w', encoding='utf-8') as fp:
            print("""\
{}

@public
def foo():
    pass

@public
class AClass:
    pass
""".format(self.import_line), file=fp)
        module = import_module('example')
        self.assertEqual(module.__all__, ['foo', 'AClass'])

    def test_atpublic_append_to_all(self):
        with open(self.modpath, 'w', encoding='utf-8') as fp:
            print("""\
__all__ = ['a', 'b']

a = 1
b = 2

{}

@public
def foo():
    pass

@public
class AClass:
    pass
""".format(self.import_line), file=fp)
        module = import_module('example')
        self.assertEqual(module.__all__, ['a', 'b', 'foo', 'AClass'])

    def test_atpublic_keywords(self):
        with open(self.modpath, 'w', encoding='utf-8') as fp:
            print("""\
{}

public(a=1, b=2)
""".format(self.import_line), file=fp)
        module = import_module('example')
        self.assertEqual(sorted(module.__all__), ['a', 'b'])

    def test_atpublic_keywords_multicall(self):
        with open(self.modpath, 'w', encoding='utf-8') as fp:
            print("""\
{}

public(b=1)
public(a=2)
""".format(self.import_line), file=fp)
        module = import_module('example')
        self.assertEqual(module.__all__, ['b', 'a'])

    def test_atpublic_keywords_global_bindings(self):
        with open(self.modpath, 'w', encoding='utf-8') as fp:
            print("""\
{}

public(a=1, b=2)
""".format(self.import_line), file=fp)
        module = import_module('example')
        self.assertEqual(module.a, 1)
        self.assertEqual(module.b, 2)

    def test_atpublic_mixnmatch(self):
        with open(self.modpath, 'w', encoding='utf-8') as fp:
            print("""\
__all__ = ['a', 'b']

a = 1
b = 2

{}

@public
def foo():
    pass

@public
class AClass:
    pass

public(c=3)
""".format(self.import_line), file=fp)
        module = import_module('example')
        self.assertEqual(module.__all__, ['a', 'b', 'foo', 'AClass', 'c'])

    def test_all_is_a_tuple(self):
        with open(self.modpath, 'w', encoding='utf-8') as fp:
            print("""\
__all__ = ('foo',)

{}

def foo():
    pass

@public
def bar():
    pass
""".format(self.import_line), file=fp)
        self.assertRaises(ValueError, import_module, 'example')


class TestPyPublic(TestPublic):
    import_line = 'from public import py_public as public'


class TestInstall(unittest.TestCase):
    @unittest.skipIf(c_public is None, 'Built without the extension module')
    def test_install_c_public(self):
        self.assertFalse(hasattr(builtins, 'public'))
        self.addCleanup(delattr, builtins, 'public')
        install()
        self.assertTrue(hasattr(builtins, 'public'))
        self.assertIs(builtins.public, c_public)

    def test_install_py_public(self):
        self.assertFalse(hasattr(builtins, 'public'))
        self.addCleanup(delattr, builtins, 'public')
        install()
        self.assertTrue(hasattr(builtins, 'public'))
        self.assertIs(builtins.public, c_public or py_public)



# Copyright (C) 2016 Barry Warsaw
#
# This project is licensed under the terms of the Apache 2.0 License.  See
# LICENSE.txt for details.

import os

from setup_helpers import get_version, require_python
from setuptools import setup, find_packages, Extension

require_python(0x30400f0)
__version__ = get_version('public/__init__.py')


# It can be a pain in some environments to build the C extension module.  For
# example, if you're using tox on a platform that doesn't have a C compiler.
# OTOH, we can be more efficient with the C extension.  In order to give
# environments the option to build what they want, we'll only build the C
# extension if this environment variable is set.

ext_modules = []
if os.getenv('ATPUBLIC_BUILD_EXTENSION', False):
    ext_modules.append(Extension('_public', ['src/public.c']))


setup(
    name='atpublic',
    version=__version__,
    description='public -- @public for populating __all__',
    long_description="""\
This is a very simple decorator and function which populates a module's
__all__ and optionally the module globals.

This provides both a pure-Python implementation and a C implementation.  It is
proposed that the C implementation be added to builtins_ for Python 3.6.
""",
    author='Barry Warsaw',
    author_email='barry@python.org',
    license='Apache 2.0',
    url='http://public.readthedocs.io/',
    packages=find_packages(),
    include_package_data=True,
    ext_modules=ext_modules,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: POSIX',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: MacOS :: MacOS X',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities',
        ]
    )
